precision lowp float;
varying highp vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;
uniform sampler2D inputImageTexture3;
uniform  float softskinRatio;
uniform  float skinwhiteRatio;
uniform  float skintuneRatio;

const vec3 CB = vec3(-37.74, -74.205, 111.945);
const vec3 CR = vec3(111.945, -93.84, -18.105);
float gauss(float x, float mean, float var)//Skin Detection Using Gaussian Mixture Models in YCbCr and Hsv Color Space
{
    float t = -0.5 * (x - mean) * (x - mean) / var;
    return exp(t);
}
float skinmap(vec3 color) //Human Skin Detection Using RGB, HSV and YCbCr Color Models
//Robust Skin Colour Detection And Tracking Algorithm
{
    float cb = dot(color, CB) + 128.0;
    float cr = dot(color, CR) + 128.0;
    float pcb = gauss(cb, 102.0, 196.0);//
    float pcr = gauss(cr, 143.0, 196.0);
    return 2.0 * pcb * pcr;
}
void main()
{
    vec4  sColor = texture2D(inputImageTexture, textureCoordinate);
    vec3 resColor=sColor.rgb;
    //skin white
    vec4 texColor = vec4(resColor.rgb, 1.0);
    float blueColor = texColor.b * 63.0;//8*8-1=63
    vec2 quad1;
    quad1.y = floor(floor(blueColor) * 0.125);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);
    vec2 quad2;
    quad2.y = floor(ceil(blueColor) * 0.125);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);
    float posOff = 0.5 / 512.0;
    float posScale = 0.125 - posOff * 2.0;
    vec2 texPos1;
    texPos1.x = (quad1.x * 0.125) + posOff + (posScale * texColor.r);
    texPos1.y = (quad1.y * 0.125) + posOff + (posScale * texColor.g);
    vec2 texPos2;
    texPos2.x = (quad2.x * 0.125) + posOff + (posScale * texColor.r);
    texPos2.y = (quad2.y * 0.125) + posOff + (posScale * texColor.g);
    vec4 newColor1 = clamp(texture2D(inputImageTexture2, texPos1), vec4(0.0), vec4(1.0));
    vec4 newColor2 = clamp(texture2D(inputImageTexture2, texPos2), vec4(0.0), vec4(1.0));
    vec4 newColor = clamp(mix(newColor1, newColor2, fract(blueColor)), vec4(0.0), vec4(1.0));
    float skinwhiteK = clamp(resColor.r, 0.0, 1.0);
    resColor.rgb = mix(resColor.rgb, newColor.rgb, skinwhiteRatio * skinwhiteK);
    resColor = clamp(resColor, 0.0, 1.0);
    // skin color
    texColor = vec4(resColor.rgb, 1.0);
    blueColor = texColor.b * 63.0;
    quad1.y = floor(floor(blueColor) * 0.125);
    quad1.x = floor(blueColor) - (quad1.y * 8.0);
    quad2.y = floor(ceil(blueColor) * 0.125);
    quad2.x = ceil(blueColor) - (quad2.y * 8.0);
    posOff = 0.5 / 512.0;
    posScale = 0.125 - posOff * 2.0;
    texPos1.x = (quad1.x * 0.125) + posOff + (posScale * texColor.r);
    texPos1.y = (quad1.y * 0.125) + posOff + (posScale * texColor.g);
    texPos2.x = (quad2.x * 0.125) + posOff + (posScale * texColor.r);
    texPos2.y = (quad2.y * 0.125) + posOff + (posScale * texColor.g);
    newColor1 = clamp(texture2D(inputImageTexture3, texPos1), vec4(0.0), vec4(1.0));
    newColor2 = clamp(texture2D(inputImageTexture3, texPos2), vec4(0.0), vec4(1.0));
    newColor = clamp(mix(newColor1, newColor2, fract(blueColor)), vec4(0.0), vec4(1.0));
    float skintureK = clamp(skinmap(resColor.rgb)*1.1, 0.0, 1.0);
    resColor.rgb = mix(resColor.rgb, newColor.rgb, skintuneRatio * skintureK);
    resColor = clamp(resColor, 0.0, 1.0);
    gl_FragColor = vec4(resColor.rgb, sColor.a);
}